

function HRAFrameHighlighter(object,textframes,color)
	object.HRAHighlightColor = color
	if object:HasScript("OnEnter") then
		object.HRAHighlightOnEnter = object:GetScript("OnEnter")
		object:SetScript("OnEnter",function()
			if this.HRAHighlightOnEnter then
				this.HRAHighlightOnEnter()
			end
			for _,frame in ipairs(textframes) do
				frame.HRAHighlightOldColor = {frame:GetTextColor()}
				frame:SetTextColor(this.HRAHighlightColor[1],this.HRAHighlightColor[2],this.HRAHighlightColor[3],this.HRAHighlightColor[4])
			end
		end)
		object.HRAHighlightOnLeave = object:GetScript("OnLeave")
		object:SetScript("OnLeave",function()
			if this.HRAHighlightOnLeave then
				this.HRAHighlightOnLeave()
			end
			for _,frame in ipairs(textframes) do
				frame:SetTextColor(frame.HRAHighlightOldColor[1],frame.HRAHighlightOldColor[2],frame.HRAHighlightOldColor[3],frame.HRAHighlightOldColor[4])
			end
		end)
	end
end

function CreateFrame_HRAEditBox(name,parent)
	local box = CreateFrame("EditBox",name,parent)
	box:SetFontObject("GameFontNormalSmall")
	box:SetAutoFocus("false")
	box:SetWidth(40)
	box:SetHeight(22)
	--frame.optionframe.intervaledit:SetPoint("TOPLEFT",frame.optionframe,"TOPLEFT",20,-60)
	box:SetScript("OnEnter",function()
		this:SetFocus()
		this:HighlightText(0)
	end)
	box:SetScript("OnLeave",function()
		this:ClearFocus()
		this:HighlightText(0,0)
	end)
	box:SetTextInsets(4,-4,0,0)
	
	box.bg = box:CreateTexture()
	box.bg:SetTexture("Interface\\AddOns\\HealingRangeAlpha\\TextBoxShort.blp")
	box.bg:SetTexCoord(0,1,0,.71)
	box.bg:SetPoint("BOTTOMLEFT",box,"BOTTOMLEFT",0,0)-- -5,-5)
	box.bg:SetPoint("TOPRIGHT",box,"TOPRIGHT",0,0)-- 5,5)
	return box
end

function HRASetColorPicker(button)
	HideUIPanel(ColorPickerFrame)
	ColorPickerFrame.parent = button  -- Inserted for my sanity
	ColorPickerFrame.func = button.swatchFunc  --  Function called by the color picker on color change
	ColorPickerFrame.hasOpacity = button.hasOpacity  --  Show the opacity slider on the colorpicker frame
	ColorPickerFrame.opacityFunc = button.opacityFunc  --  Function called by the opacity slider when you change its value
	ColorPickerFrame.opacity = button.opacity  --  Percentatge of the opacity, 1.0 is fully shown, 0 is trans
	ColorPickerFrame:SetColorRGB(button.r, button.g, button.b)
	ColorPickerFrame.previousValues = {r = button.r, g = button.g, b = button.b, opacity = button.opacity}
	ColorPickerFrame.cancelFunc = button.cancelFunc -- Function called by the colorpicker when you click the cancel button (it takes the previous values as its argument)
	ShowUIPanel(ColorPickerFrame)
end

function HRASetButtonColor(button,color)
	if color == nil then
		color = {r=1,g=1,b=1,opacity=1}
	end
	button.r = color.r
	button.g = color.g
	button.b = color.b
	button.opacity = 1-color.opacity
	local x = button:GetNormalTexture()
	x:SetVertexColor(button.r, button.g, button.b)
end

function HRAGetButtonColor(button)
	return {r=button.r,g=button.g,b=button.b,opacity=1-button.opacity}
end

function CreateFrame_HRAColorButton(name,parent)
	local button = CreateFrame("Button",name,parent)
	button:SetWidth(20)
	button:SetHeight(20)
	button:SetNormalTexture("Interface\\ChatFrame\\ChatFrameColorSwatch")
	
	button.r = 1
	button.g = 1
	button.b = 1
	button.opacity = 0
	button.hasOpacity = true
	button.opacityFunc = nil
	button.swatchFunc = nil
	button.cancelFunc = nil
	
	button.bg = button:CreateTexture(button:GetName().."SwatchBg","BACKGROUND")
	button.bg:SetWidth(17)
	button.bg:SetHeight(17)
	button.bg:SetPoint("CENTER",button,"CENTER")
	button.bg:SetTexture(1,1,1)
	button.bg:SetVertexColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	
	button:SetScript("OnClick",function() HRASetColorPicker(this) end )
	button:SetScript("OnEnter",function()
		this.bg:SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end )
	button:SetScript("OnLeave",function()
		this.bg:SetVertexColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	end )
	
	return button
end

local CreateFrame_HRAEnableCheck = function(name,parent)
	local checkbutton = CreateFrame("CheckButton",name,parent,"UICheckButtonTemplate")
	checkbutton.text = checkbutton:CreateFontString(checkbutton:GetName().."Text","ARTWORK","GameFontNormalSmall")
	checkbutton.text:SetPoint("LEFT",checkbutton,"RIGHT")
	return checkbutton
end

local CreateFrame_HRASliderTemplate = function (name,parent)
	local slider = CreateFrame("Slider",name,parent)
	slider:SetOrientation("HORIZONTAL")
	slider:EnableMouse("true")
	slider:SetWidth(140)
	slider:SetHeight(15)
	slider:SetHitRectInsets(0,0,-10,-10)
	slider:SetMinMaxValues(0,1)
	slider:SetValueStep(0.1)
	slider:SetBackdrop({bgFile="Interface\\Buttons\\UI-SliderBar-Background",
						edgeFile="Interface\\Buttons\\UI-SliderBar-Border",
						tile="true",
						tileSize = 8,
						edgeSize = 8, 
						insets={left=3,right=3,top=6,bottom=6}})
	
	slider.current = slider:CreateFontString(slider:GetName().."Current","ARTWORK","GameFontNormalSmall")
	slider.current:SetPoint("LEFT",slider,"RIGHT",5,0)
	slider.low = slider:CreateFontString(slider:GetName().."Low","ARTWORK","GameFontNormalSmall")
	slider.low:SetPoint("TOPLEFT",slider,"BOTTOMLEFT",2,3)
	slider.low:SetText("0%")
	slider.high = slider:CreateFontString(slider:GetName().."High","ARTWORK","GameFontNormalSmall")
	slider.high:SetPoint("TOPRIGHT",slider,"BOTTOMRIGHT",-2,3)
	slider.high:SetText("100%")
	
	slider:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
	
	slider:SetScript("OnValueChanged",function()
		local x=this:GetName()
		this.current:SetText(math.floor(this:GetValue()*100).."%")
	end)
		
	return slider
end

local frame = CreateFrame("Frame","HRAFrame",UIParent)

frame.optionframe = CreateFrame("Frame","HRAOptionFrame",frame)
frame.optionframe:Hide()
frame.optionframe:SetToplevel("true")
frame.optionframe:SetWidth(230)
frame.optionframe:SetHeight(360)
frame.optionframe:SetMovable("true")
frame.optionframe:EnableMouse("true")
frame.optionframe:SetPoint("CENTER",UIParent,"CENTER")
frame.optionframe:SetBackdrop({bgFile="Interface\\ChatFrame\\ChatFrameBackground",
							edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
							tile="true",
							tileSize = 16,
							edgeSize = 16, 
							insets={left=4,right=4,top=4,bottom=4}})
frame.optionframe:SetBackdropColor(0,.6,0,.5)

frame.optionframe.text = frame.optionframe:CreateFontString(frame.optionframe:GetName().."Text","ARTWORK","GameFontNormalLarge")
frame.optionframe.text:SetPoint("TOP",frame.optionframe,"TOP",0,-10)
frame.optionframe.text:SetText("Healing Range Alpha")

frame.optionframe.enablecheck = CreateFrame_HRAEnableCheck(frame.optionframe:GetName().."EnableCheck",frame.optionframe)
frame.optionframe.enablecheck:SetPoint("TOPLEFT",frame.optionframe,"TOPLEFT",5,-30)
frame.optionframe.enablecheck.text = frame.optionframe.enablecheck:CreateFontString(frame.optionframe.enablecheck:GetName().."Text","ARTWORK","GameFontNormalSmall")
frame.optionframe.enablecheck.text:SetPoint("LEFT",frame.optionframe.enablecheck,"RIGHT")
frame.optionframe.enablecheck.text:SetText("Enable HRA")
HRAFrameHighlighter(frame.optionframe.enablecheck,{frame.optionframe.enablecheck.text},{1,1,1,1})

frame.optionframe.debugcheck = CreateFrame_HRAEnableCheck(frame.optionframe:GetName().."DebugCheck",frame.optionframe)
frame.optionframe.debugcheck:SetPoint("TOPRIGHT",frame.optionframe,"TOPRIGHT",-5,-30)
frame.optionframe.debugcheck.text = frame.optionframe.debugcheck:CreateFontString(frame.optionframe.debugcheck:GetName().."Text","ARTWORK","GameFontNormalSmall")
frame.optionframe.debugcheck.text:SetPoint("RIGHT",frame.optionframe.debugcheck,"LEFT")
frame.optionframe.debugcheck.text:SetText("Debug")
HRAFrameHighlighter(frame.optionframe.debugcheck,{frame.optionframe.debugcheck.text},{1,1,1,1})

frame.optionframe.intervaledit = CreateFrame_HRAEditBox(frame.optionframe:GetName().."IntervalEdit",frame.optionframe)
frame.optionframe.intervaledit:SetMaxLetters(4)
frame.optionframe.intervaledit:SetNumeric("true")
frame.optionframe.intervaledit:SetPoint("TOPLEFT",frame.optionframe,"TOPLEFT",15,-65)

frame.optionframe.intervaledit.text = frame.optionframe.intervaledit:CreateFontString(frame.optionframe.intervaledit:GetName().."Text","ARTWORK","GameFontNormalSmall")
frame.optionframe.intervaledit.text:SetPoint("LEFT",frame.optionframe.intervaledit,"RIGHT",0,5)
frame.optionframe.intervaledit.text:SetText("(ms) interval updates")

HRAFrameHighlighter(frame.optionframe.intervaledit,{frame.optionframe.intervaledit.text},{1,1,1,1})

frame.optionframe.depthedit = CreateFrame_HRAEditBox(frame.optionframe:GetName().."DepthEdit",frame.optionframe)
frame.optionframe.depthedit:SetMaxLetters(1)
frame.optionframe.depthedit:SetNumeric("true")
frame.optionframe.depthedit:SetPoint("TOPRIGHT",frame.optionframe,"TOPRIGHT",-15,-80)

frame.optionframe.depthedit.text = frame.optionframe.intervaledit:CreateFontString(frame.optionframe.intervaledit:GetName().."Text","ARTWORK","GameFontNormalSmall")
frame.optionframe.depthedit.text:SetPoint("RIGHT",frame.optionframe.depthedit,"LEFT",0,-5)
frame.optionframe.depthedit.text:SetText("Colorize depth levels")
HRAFrameHighlighter(frame.optionframe.depthedit,{frame.optionframe.depthedit.text},{1,1,1,1})

frame.optionframe.sliders = {}

function HRA_insertRange(text,range,default)
	local id
	for k,v in pairs(frame.optionframe.sliders) do
		if not v:IsShown() then
			id = k
			frame.optionframe.sliders[id]:Show()
		end
	end
	if id == nil then
		id = table.getn(frame.optionframe.sliders)+1
		frame.optionframe.sliders[id] = CreateFrame_HRASliderTemplate(frame.optionframe:GetName().."RangeSlider"..id,frame.optionframe)
		frame.optionframe:SetHeight(130+(id*45)+30)
	end
	
	frame.optionframe.sliders[id]:SetPoint("TOPLEFT",frame.optionframe,"TOPLEFT",10,-90-(id*45)-200)
	frame.optionframe.sliders[id]:SetValue(default)
	
	frame.optionframe.sliders[id].range = tonumber(range)
	
	frame.optionframe.sliders[id].description = frame.optionframe.sliders[id]:CreateFontString(frame.optionframe.sliders[id]:GetName().."Description","ARTWORK","GameFontNormalSmall")
	frame.optionframe.sliders[id].description:SetText(text)
	frame.optionframe.sliders[id].description:SetPoint("BOTTOMLEFT",frame.optionframe.sliders[id],"TOPLEFT",0,5)
	
	
	frame.optionframe.sliders[id].color = CreateFrame_HRAColorButton(frame.optionframe.sliders[id]:GetName().."Color",frame.optionframe.sliders[id])
	frame.optionframe.sliders[id].color:SetPoint("LEFT",frame.optionframe.sliders[id],"RIGHT",50,0)
	frame.optionframe.sliders[id].color.opacityFunc = function()
		if this:GetObjectType() == "Slider" then
			ColorPickerFrame.parent.opacity = this:GetValue()
		end
	end
	frame.optionframe.sliders[id].color.swatchFunc = function()
		if this:GetObjectType() == "ColorSelect" then
			ColorPickerFrame.parent.r,ColorPickerFrame.parent.g,ColorPickerFrame.parent.b = this:GetColorRGB()
			local x = ColorPickerFrame.parent:GetNormalTexture()
			x:SetVertexColor(ColorPickerFrame.parent.r, ColorPickerFrame.parent.g, ColorPickerFrame.parent.b)
		end
	end
	frame.optionframe.sliders[id].color.cancelFunc = function(previous)
		ColorPickerFrame.parent.r = previous.r
		ColorPickerFrame.parent.g = previous.g
		ColorPickerFrame.parent.b = previous.b
		ColorPickerFrame.parent.opacity = previous.opacity
		local x = ColorPickerFrame.parent:GetNormalTexture()
		x:SetVertexColor(ColorPickerFrame.parent.r, ColorPickerFrame.parent.g, ColorPickerFrame.parent.b)
	end
	
	HRAFrameHighlighter(frame.optionframe.sliders[id],{frame.optionframe.sliders[id].description},{1,1,1,1})
	HRAFrameHighlighter(frame.optionframe.sliders[id].color,{frame.optionframe.sliders[id].description},{1,1,1,1})
end

function HRA_removeRanges()
	for k,v in pairs(frame.optionframe.sliders) do
		if v.range ~= 28 or v.range ~= 100 or v.range ~= 30000 then
			v:Hide()
		end
	end
end

function HRA_sortRanges()
	table.sort( frame.optionframe.sliders, function(l,r) return (l.range > r.range or not l:IsShown()) end )
	
	for i=1,table.getn(frame.optionframe.sliders) do
		frame.optionframe.sliders[i]:SetPoint("TOPLEFT",frame.optionframe,"TOPLEFT",10,-75-(i*45))
	end
end

frame.optionframe.applybutton = CreateFrame("Button",frame.optionframe:GetName().."Apply",frame.optionframe,"OptionsButtonTemplate")
frame.optionframe.applybutton:SetPoint("BOTTOMRIGHT",frame.optionframe,"BOTTOMRIGHT",-10,10)
frame.optionframe.applybutton:SetText("Apply")
frame.optionframe.applybutton:SetScript("OnClick",function()
	HRA_ApplyChanges()
	HRAOptionFrame:Hide()
end)

frame.optionframe.cancelbutton = CreateFrame("Button",frame.optionframe:GetName().."Cancel",frame.optionframe,"OptionsButtonTemplate")
frame.optionframe.cancelbutton:SetPoint("BOTTOMLEFT",frame.optionframe,"BOTTOMLEFT",10,10)
frame.optionframe.cancelbutton:SetText("Cancel")
frame.optionframe.cancelbutton:SetScript("OnClick",function()
	HRAOptionFrame:Hide()
end)

frame.optionframe.resetbutton = CreateFrame("Button",frame.optionframe:GetName().."Reset",frame.optionframe,"OptionsButtonTemplate")
frame.optionframe.resetbutton:SetPoint("BOTTOM",frame.optionframe,"BOTTOM",0,40)
frame.optionframe.resetbutton:SetText("Reset")
frame.optionframe.resetbutton:SetScript("OnClick",function()
	HRA_defaults()
	HRA_LoadSettings()
end)

frame.optionframe:SetScript("OnShow",function()
	HRA_LoadSettings()
end)
frame.optionframe:SetScript("OnMouseDown",function()
	this:StartMoving()
end)
frame.optionframe:SetScript("OnMouseUp",function()
	this:StopMovingOrSizing()
end)

local tooltip = CreateFrame("GameTooltip","HRATooltip",UIParent,"GameTooltipTemplate")
tooltip:SetScript("Onload",function()
	this:SetOwner(this, "ANCHOR_NONE")
end)

frame:SetScript("OnEvent",function()
	HRA_OnEvent(event)
end)
frame:SetScript("OnUpdate",function()
	HRA_OnUpdate(arg1)
end)